package itemchunkdemo;

import javax.sql.DataSource;
import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.batch.core.*;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith( SpringJUnit4ClassRunner.class )
@ContextConfiguration( classes = MeineApplication.class )
public class ItemChunkJobTest
{
   @Autowired private JobLauncher jobLauncher;
   @Autowired private Job job;
   private JdbcTemplate jdbcTemplate;

   @Autowired
   public void setDataSource( DataSource dataSource )
   {
      this.jdbcTemplate = new JdbcTemplate( dataSource );
   }

   @Test
   public void testJob() throws Exception
   {
      // Joblauf ohne Job-Parameter:
      JobExecution je1 = jobLauncher.run( job, new JobParameters() );
      Assert.assertEquals( BatchStatus.COMPLETED, je1.getStatus() );
      Assert.assertEquals( 0, je1.getAllFailureExceptions().size() );
      Assert.assertEquals( 1, je1.getStepExecutions().size() );
      for( StepExecution se : je1.getStepExecutions() ) {
         Assert.assertEquals( BatchStatus.COMPLETED, se.getStatus() );
         // 6 Datensaetze, 1 Chunk-Commit:
         Assert.assertEquals( 6, se.getReadCount() );
         Assert.assertEquals( 6, se.getWriteCount() );
         Assert.assertEquals( 1, se.getCommitCount() );
      }
      // Joblauf mit Job-Parameter zum Input-Datei-Pfad:
      JobExecution je2 = jobLauncher.run( job,
            new JobParametersBuilder().addString( "InputDateiPfad", "src/main/resources/BeispielDaten2.csv" ).toJobParameters() );
      Assert.assertEquals( BatchStatus.COMPLETED, je2.getStatus() );
      Assert.assertEquals( 0, je2.getAllFailureExceptions().size() );
      Assert.assertEquals( 1, je2.getStepExecutions().size() );
      for( StepExecution se : je2.getStepExecutions() ) {
         Assert.assertEquals( BatchStatus.COMPLETED, se.getStatus() );
         // 1 Datensatz, 1 Chunk-Commit:
         Assert.assertEquals( 1, se.getReadCount() );
         Assert.assertEquals( 1, se.getWriteCount() );
         Assert.assertEquals( 1, se.getCommitCount() );
      }
      // Ueberpruefe Ergebnis in DB (6 + 1 = 7 Datensaetze):
      Assert.assertEquals( Integer.valueOf(  7 ), jdbcTemplate.queryForObject( "Select count(*) from Meinedaten", Integer.class ) );
      Assert.assertEquals( Integer.valueOf( 70 ), jdbcTemplate.queryForObject( "Select sum(buchstabenanzahl) from Meinedaten", Integer.class ) );
   }
}
