package conditionalflowdemo;

import org.springframework.batch.core.*;
import org.springframework.batch.core.configuration.annotation.*;
import org.springframework.batch.core.scope.context.ChunkContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.*;

@Configuration
@EnableBatchProcessing
public class MeineJobConfiguration
{
   public static final String OK_ODER_FEHLER = "OK_ODER_FEHLER";
   @Autowired private JobBuilderFactory  jobBuilderFactory;
   @Autowired private StepBuilderFactory stepBuilderFactory;

   /** Tasklet: Zeigt Text im Kommandozeilenfenster */
   public class PrintTextTasklet implements Tasklet
   {
      final String text;

      public PrintTextTasklet( String text ) { this.text = text; }

      @Override
      public RepeatStatus execute( StepContribution sc, ChunkContext cc ) throws Exception {
         System.out.println( text );
         return RepeatStatus.FINISHED;
      }
   }

   /** Tasklet: "Geschaeftsprozess", entweder erfolgreich oder mit Fehler */
   public class ArbeitsTasklet extends PrintTextTasklet
   {
      public ArbeitsTasklet( String text ) { super( text ); }

      @Override
      public RepeatStatus execute( StepContribution sc, ChunkContext cc ) throws Exception {
         String okOderFehler = cc.getStepContext().getStepExecution().getJobParameters().getString( OK_ODER_FEHLER );
         if( ( okOderFehler != null ) ? !okOderFehler.equalsIgnoreCase( "ok" ) : (Math.random() < 0.5) ) {
            System.out.println( super.text + ": mit Fehler" );
            throw new Exception( "Fehler" );
         }
         System.out.println( super.text + ": ok" );
         return RepeatStatus.FINISHED;
      }
   }

   @Bean
   public Step arbeitsStep()
   {
      return stepBuilderFactory.get( "arbeitsStep" ).tasklet( new ArbeitsTasklet( "ArbeitsStep" ) ).build();
   }

   @Bean
   public Step fehlerbehandlungsStep()
   {
      return stepBuilderFactory.get( "fehlerbehandlungsStep" ).tasklet( new PrintTextTasklet( "FehlerbehandlungsStep" ) ).build();
   }

   @Bean
   public Step okStep()
   {
      return stepBuilderFactory.get( "okStep" ).tasklet( new PrintTextTasklet( "OkStep" ) ).build();
   }

   @Bean
   public Step abschliessenderStep()
   {
      return stepBuilderFactory.get( "abschliessenderStep" ).tasklet( new PrintTextTasklet( "AbschliessenderStep" ) ).build();
   }

   /** Job: fuehrt Steps aus */
   @Bean
   public Job meinConditionalFlowJob()
   {
      return jobBuilderFactory.get( "meinConditionalFlowJob" )
             .flow( arbeitsStep() ).on( "FAILED" ).to( fehlerbehandlungsStep() ).next( abschliessenderStep() )
             .from( arbeitsStep() ).on( "*" ).to( okStep() ).next( abschliessenderStep() )
             .end().build();
   }
}
