package conditionalflowdemo;

import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.batch.core.*;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@ContextConfiguration( classes = MeineApplication.class )
@RunWith( SpringJUnit4ClassRunner.class )
public class BatchApplicationTest
{
   @Autowired private JobLauncher jobLauncher;
   @Autowired private Job job;

   @Test
   public void testJobOk() throws Exception
   {
      System.out.println( "\nJoblauf mit Job-Parameter 'ok':" );
      JobExecution je = jobLauncher.run( job, new JobParametersBuilder().addString(
            MeineJobConfiguration.OK_ODER_FEHLER, "ok" ).toJobParameters() );
      // Ueberpruefe Job-Ergebnis:
      String[] steps = new String[] { "arbeitsStep", "okStep", "abschliessenderStep" };
      Assert.assertEquals( BatchStatus.COMPLETED, je.getStatus() );
      Assert.assertEquals( 0, je.getAllFailureExceptions().size() );
      Assert.assertEquals( 3, je.getStepExecutions().size() );
      int i = 0;
      for( StepExecution se : je.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": CommitCount = " + se.getCommitCount() +
                             ", Status = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() +
                             ", StepName = " + se.getStepName() );
         Assert.assertEquals( BatchStatus.COMPLETED, se.getStatus() );
         Assert.assertEquals( 1, se.getCommitCount() );
         Assert.assertEquals( steps[i++], se.getStepName() );
      }
   }

   @Test
   public void testJobMitFehler() throws Exception
   {
      System.out.println( "\nJoblauf mit Job-Parameter 'Fehler':" );
      JobExecution je = jobLauncher.run( job, new JobParametersBuilder().addString(
            MeineJobConfiguration.OK_ODER_FEHLER, "Fehler" ).toJobParameters() );
      // Ueberpruefe Job-Ergebnis:
      String[]      steps   = new String[]      { "arbeitsStep", "fehlerbehandlungsStep", "abschliessenderStep" };
      BatchStatus[] baStati = new BatchStatus[] { BatchStatus.ABANDONED, BatchStatus.COMPLETED, BatchStatus.COMPLETED };
      ExitStatus[]  exStati = new ExitStatus[]  { ExitStatus.FAILED, ExitStatus.COMPLETED, ExitStatus.COMPLETED };
      int[]         commits = new int[] { 0, 1, 1 };
      Assert.assertEquals( BatchStatus.COMPLETED, je.getStatus() );
      Assert.assertEquals( 1, je.getAllFailureExceptions().size() );
      Assert.assertEquals( 3, je.getStepExecutions().size() );
      int i = 0;
      for( StepExecution se : je.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": CommitCount = " + se.getCommitCount() +
                             ", Status = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() +
                             ", StepName = " + se.getStepName() );
         Assert.assertEquals( commits[i],               se.getCommitCount() );
         Assert.assertEquals( baStati[i],               se.getStatus() );
         Assert.assertEquals( exStati[i].getExitCode(), se.getExitStatus().getExitCode() );
         Assert.assertEquals( steps[i++],               se.getStepName() );
      }
   }
}
