package springbatchdemo;

import java.text.SimpleDateFormat;
import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.batch.core.*;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith( SpringJUnit4ClassRunner.class )
@ContextConfiguration( classes = MeineApplication.class )
public class SplitFlowJobTest
{
   @Autowired private JobLauncher jobLauncher;
   @Autowired private Job job;
   private SimpleDateFormat df = new SimpleDateFormat( "HH:mm:ss.SSS" );

   @Test
   public void testSplitFlowJob() throws Exception
   {
      System.out.println( "\nDie drei Steps step3, step4 und step5 werden im 'Split Flow' parallel ausgefuehrt:" );
      JobExecution je = jobLauncher.run( job, new JobParameters() );
      System.out.println( "" );
      // Ueberpruefe Job-Ergebnis:
      Assert.assertEquals( BatchStatus.COMPLETED, je.getStatus() );
      Assert.assertEquals( ExitStatus.COMPLETED,  je.getExitStatus() );
      Assert.assertEquals( 0, je.getAllFailureExceptions().size() );
      Assert.assertEquals( 7, je.getStepExecutions().size() );
      double dauerSum = 0.;
      for( StepExecution se : je.getStepExecutions() ) {
         Assert.assertEquals( BatchStatus.COMPLETED, se.getStatus() );
         Assert.assertEquals( ExitStatus.COMPLETED,  se.getExitStatus() );
         Assert.assertEquals( 1, se.getCommitCount() );
         double dauer = ((double) ((se.getEndTime().getTime() - se.getStartTime().getTime()) / 100)) / 10;
         dauerSum += dauer;
         System.out.println( se.getStepName() + ": von " + df.format( se.getStartTime() ) +
                                                 " bis " + df.format( se.getEndTime() ) +
                                                 " h, Dauer: " + dauer + " s" );
      }
      System.out.println( "Summe der Steps: " + dauerSum + " s" );
      double dauerJob = ((double) ((je.getEndTime().getTime() - je.getStartTime().getTime()) / 100)) / 10;
      System.out.println( "Gesamtdauer Job: " + dauerJob + " s" );
      Assert.assertTrue( dauerSum > dauerJob );
   }
}
